;;########################################################################
;; anovavs2.lsp
;; Copyright (c) 1993-96 by Forrest W. Young
;; anovavis.lsp and anovavs2.lsp contain the code for anova visualization.
;;########################################################################

(require "vista")

(defproto change-y-axis-overlay-proto '(x1 y1 x2 y2 location) '()
  graph-overlay-proto)

(defmeth change-y-axis-overlay-proto :location (&optional (list nil set)) 
  (when set (setf (slot-value 'location) list)) 
  (slot-value 'location))

(defun make-yaxis-overlay (graph)
  (let* ((overlay (send change-y-axis-overlay-proto :new))
         (ascent (send graph :text-ascent))
         (h (+ ascent (send graph :text-descent)))
         (x ascent)
         (y (round (* 1.5 ascent)))
         (box ascent))
  (send graph :margin 0 (round (* 1.5 h)) 0 0)
  (send overlay :location (list x y box (round (+ x (* 1.5 box)))))
  overlay))

(defmeth change-y-axis-overlay-proto :redraw ()
  (let* ((loc (send self :location))
         (x (first loc))
         (y (second loc))
         (box (third loc))
         (string-x (fourth loc))
         (graph (send self :graph)))
    (send graph :frame-rect x (- y box) box box)
    (send graph :draw-string "Y-Axis" string-x y)))

(defmeth change-y-axis-overlay-proto :do-click (x y m1 m2) 
  (let* ((loc (send self :location))
         (box (third loc))
         (left (first loc))
         (top (- (second loc) box))
         (right (+ left box))
         (bottom (+ top box))
         (graph (send self :graph))) 
    (when (and (< left x right) (< top y bottom))
          (send graph :change-y-axis)
          t)))


; ************ Leverage Plot ***********************************


(defun leverage-plot 
  (sel model &key (title "Regression Plot")
     (variable-labels (list "Predicted Values" "Values"))  
     point-labels location size y-range y-axis y-mean (go-away t) (show t))
  (send leverage-plot-proto :new sel model
        :title title :point-labels point-labels
        :variable-labels variable-labels 
        :location location  :size size 
        :y-range y-range :y-axis y-axis :y-mean y-mean
        :go-away go-away :show show))

(defun lplot (sel model &rest args)
  (apply #'leverage-plot sel model args))

(defproto leverage-plot-proto '(confidence-constant) () scatterplot-proto)

(defmeth leverage-plot-proto :isnew 
  (sel model &key title 
         variable-labels point-labels location size 
         y-range y-axis y-mean go-away show)
  (call-next-method 2 :title title :variable-labels variable-labels
                    :location location :size size :go-away go-away :show nil)
  (send self :confidence-constant 
        (send self :make-confidence-constant model))
  (send self :new-menu "RegPlot" 
              :items '(LINK SHOWING-LABELS MOUSE RESIZE-BRUSH DASH 
                            ERASE-SELECTION FOCUS-ON-SELECTION SHOW-ALL 
                            SYMBOL COLOR SELECTION))
  (send self :new-plot sel model 
        :variable-labels variable-labels
        :point-labels point-labels 
        :y-range y-range 
        :y-axis y-axis
        :y-mean y-mean)
  (when show (send self :show-window))
  self)

(defmeth leverage-plot-proto :confidence-constant 
  (&optional (number nil set))
"Method args: (&optional number)
Sets or returns constant for calculating confidence envelope."
  (if set (setf (slot-value 'confidence-constant) number))
  (slot-value 'confidence-constant))

(defmeth leverage-plot-proto :new-plot 
  (sel model &key variable-labels point-labels y-range y-axis y-mean)
  (let* ((upper nil)
         (lower nil)
         (nway (send model :nway-model))
         (x (send nway :x))
         (y (send nway :y))
         (s (standard-deviation y))
         (t-value nil))
    (send self :start-buffering)
    (send self :clear)
    (when (not (= sel 0))
          (send self :title "Partial Regression Plot")
          (send self :add-points 
                (combine (send model :grouped-values 
                               (select (send model :lev sel) 0) 
                               model (1- sel)))
                (combine (send model :grouped-values 
                               (select (send model :lev sel) 1) 
                               model (1- sel))))
          (send self :variable-label 0 
                (strcat (select (send (send model :nway-model) :source-names) 
                                (1- sel)) " Leverage")))
    (when (= sel 0)
          (send self :title "Regression Plot")
          (send self :add-points 
                (select (send model :lev sel) 0)
                (select (send model :lev sel) 1))
          (send self :variable-label 0 
                (strcat "Predicted " (select (send model :variables) 0))))
    (send self :variable-label 1 (select (send model :variables) 0))
    (send self :range 0 (first y-range) (second y-range))
    (apply #'send self :x-axis y-axis)
    (send self :range 1 (first y-range) (second y-range))
    (apply #'send self :y-axis y-axis)
    (send self :confidence-envelope model sel)
    (send self :abline y-mean 0)
    (send self :abline 0 1)
    (send self :redraw)
    (send self :buffer-to-screen)
    ))

(defmeth leverage-plot-proto :make-confidence-constant (model)
"Args: Model
Calculates constant for calculating confidence interval."
  (let* ((nway-model (send model :nway-model))
         (s-sq (variance (combine (send model :data))))
         (t-sq (t-quant .975 (1- (send nway-model :num-cases))))
         (x (send model :leverages))
         (x-bar (matrix (list 1 (second (size x)))
                        (mapcar #'mean (column-list x))))
         (xpxi (inverse (matmult (transpose x) x)))
         (h-bar (select (matmult x-bar xpxi (transpose x-bar)) 0 0))
         )
    (* s-sq t-sq h-bar)))
  
(defmeth leverage-plot-proto :confidence-envelope (model sel)
  (let* ((nway-model (send model :nway-model))
         (df (send nway-model :source-degrees-of-freedom))
         (ms (/ (send nway-model :source-sum-of-squares) df))
         (f-obs  (/ (select ms sel) (first (last ms))))
         (f-crit (f-quant .95 (select df sel) (first (last df))))
         (f-ratio (/ f-crit f-obs))
         (x-range (send self :range 0))
         (lev-mean (mean (select (send model :lev sel) 0)))
         (half-range (max (- (second x-range) lev-mean) 
                          (- lev-mean (first x-range))))
         (sequence (* half-range (/ (^ (iseq 9) 4) (^ 8 4))))
         (z (combine (reverse (- lev-mean sequence)) 
                     (rest (+ lev-mean sequence))))
         (ffz-sq (* f-ratio (^ (- z lev-mean) 2)))
         (boundary-line (sqrt (+ (send self :confidence-constant) ffz-sq))))
    (send self :add-lines z (- z boundary-line) )
    (send self :add-lines z (+ z boundary-line) )
    ))
