;; crspmob2.lsp
;; contains code to implement visualizations for correspondence analysis.
;; Copyright (c) 1992-94 by Lee Bee Leng. Modified May 31, 1995 by FWY

;-----------------
;  Visualization
;-----------------                               

;forrest added next line
(require "vista")
 
(defmeth Corresp-proto :CRS-Spin-Plot ()
  (let* ((NR (send self :NRows))
         (NC (send self :NCols))
         (Coordinates nil))
    (if (not (send current-model :Point-Moved-p))
        (setf Coordinates (bind-rows (send self :norm-rows)
                                     (send self :norm-cols)))
        (setf Coordinates (bind-rows (send self :o-norm-rows)
                                     (send self :o-norm-cols))))
    (setf Spinplot (spin-plot (column-list Coordinates)
                              :show nil
                              :size graph-size
                              :location location11
                              :title "Spinplot"
                              :variable-labels 
                              (combine 
                               '("Dim1" "Dim2" "Dim3")
                               (repeat "" (- (send self :maxdim) 3)))
                              :depth-cuing nil))
    (send Spinplot :mouse-mode 'hand-rotate)
    (send Spinplot :use-color t)
    (send Spinplot :point-color (iseq NR) 'blue)
    (send Spinplot :point-symbol (iseq NR) 'cross)
    (send Spinplot :showing-labels t)
    (send Spinplot :point-label (iseq NR) (send self :labels))
    (send Spinplot :point-label (+ NR (iseq Nc)) (send self :variables))
    (send Spinplot :linked t)
    (defmeth Spinplot :close ()
      (send self :remove)
      (send Scatterplot :remove)
      (send Rows-&-Cols :remove)
      (send Residual-plot :remove)
      (send Fit-plot :remove)
      (send Dimension-list :remove))
    (send Spinplot :show-window)))
 
 
(defmeth Corresp-proto :CRS-Scatter-Plot ()
  (let* ((NR (send self :NRows))
         (NC (send self :NCols))
         (axis-labels (mapcar #'(lambda (x) 
                                  (format nil "Dimension ~a" x))
                              (+ 1 (iseq (send self :maxdim)))))
         (Coordinates nil))
    (if (not (send current-model :Point-Moved-p))
        (setf Coordinates (bind-rows (send self :norm-rows)
                                     (send self :norm-cols)))
        (setf Coordinates (bind-rows (send self :o-norm-rows)
                                     (send self :o-norm-cols))))
    (setf Scatterplot (plot-points (column-list Coordinates)
                                   :show nil
                                   :size graph-size
                                   :location location12
                                   :title "Scatterplot"
                                   :variable-labels axis-labels))
    (send Scatterplot :mouse-mode 'brushing)
    (send Scatterplot :use-color t)
    (send Scatterplot :point-color (iseq NR) 'blue)
    (send Scatterplot :point-symbol (iseq NR) 'cross)
    (send Scatterplot :showing-labels t)
    (send Scatterplot :point-label (iseq NR) (send self :labels))
    (send Scatterplot :point-label (+ NR (iseq Nc)) (send self :variables))
    (send Scatterplot :linked t)
    (send Scatterplot :x-axis t t 2)
    (send Scatterplot :y-axis t t 2)
    (send Scatterplot :show-window)
    (if (/= (send self :norm-opt) 2)
        (send Scatterplot :add-mouse-mode 'Move-Point
              :title "Point-Moving Mode"
              :cursor 'finger
              :click :do-point-moving))
 
    (defmeth Scatterplot :do-point-moving (x y m1 m2)
      (let* ((pt (send self :drag-point x y :draw nil))
             (var1 (first (send self :current-variables)))
             (var2 (second (send self :current-variables)))
             (n (send current-model :Nrows))
             (a nil)
             (b nil))
        (when pt
              (send current-model :Point-Moved-p t)
              (cond 
                ((or (and (= (send current-model :Norm-Opt) 0)
                          (<= pt (- n 1)))
                     (and (= (send current-model :Norm-Opt) 1)
                          (> pt (- n 1))))
                 (send self :redraw-content)
                 (setf a (send self :point-coordinate var1 pt))
                 (setf b (send self :point-coordinate var2 pt))
                 (send Scatterplot :showing-labels nil)
                 (send Spinplot :showing-labels nil)
                 (send Rows-&-Cols :unselect-all-points)
                 (send Spinplot :point-coordinate var1 pt a)
                 (send Spinplot :point-coordinate var2 pt b)
                 (send Spinplot :redraw t)
                 (send Spinplot :point-color pt 'red)
                 (send Scatterplot :point-color pt 'red)
                 (send Rows-&-Cols :point-color pt 'red)
                 (send Rows-&-Cols :point-state pt 'selected)
                 (send self :Reset-Coordinates pt a b)
                 (send self :Reset-Residuals)
                 (send self :Reset-Fitplot pt))
                (t
                 (cond 
                   ((<= pt (- n 1))
                    (send self :point-coordinate var1 pt
                          (aref (send current-model :norm-rows)
                                pt var1))
                    (send self :point-coordinate var2 pt
                          (aref (send current-model :norm-rows)
                                pt var2)))
                   (t
                    (send self :point-coordinate var1 pt
                          (aref (send current-model :norm-cols)
                                (- pt n) var1))
                    (send self :point-coordinate var2 pt
                          (aref (send current-model :norm-cols)
                                (- pt n) var2)))))))))
 
    (defmeth Scatterplot :Reset-Coordinates (pt a b)
      (let* ((n (send current-model :Nrows))
             (m (send current-model :Ncols))
             (row-mass (send current-model :row-mass))
             (col-mass (send current-model :col-mass))
             (F (send current-model :norm-rows))
             (num-rows-F (first (array-dimensions F)))
             (G (send current-model :norm-cols))
             (num-rows-G (first (array-dimensions G)))
             (vars (send self :current-variables)))
        (cond
          ((<= pt (- n 1)) #|row point moved|#
           (setf (aref F pt (first vars)) a)
           (setf (aref F pt (second vars)) b)
           (setf G (transpose (%* (inverse (%* (transpose F) F))
                                  (transpose F)
                                  (inverse (diagonal row-mass))
                                  (send current-model :table)
                                  (inverse (diagonal col-mass)))))
           (dotimes (i (send current-model :maxdim))
                    (send Scatterplot :point-coordinate 
                          i (iseq n (- (+ n m) 1))
                          (combine (select G (iseq num-rows-G) i)))
                    (send Spinplot :point-coordinate 
                          i (iseq n (- (+ n m) 1))
                          (combine (select G (iseq num-rows-G) i))))
           (when (not (send self :Hide-Cols))
                 (send self :adjust-to-data :draw nil)
                 (send self :x-axis t t 2)
                 (send self :y-axis t t 2))
           (send Spinplot :redraw t)
           (send Spinplot :adjust-to-data))
          ((> pt (- n 1)) #|column point moved|#
           (setf (aref G (- pt n) (first vars)) a)
           (setf (aref G (- pt n) (second vars)) b)
           (setf F (%* (inverse (diagonal row-mass))
                       (send current-model :table)
                       (inverse (diagonal col-mass))
                       G
                       (inverse (%* (transpose G) G))))
           (dotimes (i (send current-model :maxdim))
                    (send Scatterplot :point-coordinate 
                          i (iseq n)
                          (combine (select F (iseq num-rows-F) i)))
                    (send Spinplot :point-coordinate 
                          i (iseq n)
                          (combine (select F (iseq num-rows-F) i))))
           (when (not (send self :Hide-Rows))
                 (send self :adjust-to-data :draw nil)
                 (send self :x-axis t t 2)
                 (send self :y-axis t t 2))
           (send Spinplot :redraw t)
           (send Spinplot :adjust-to-data)))
        (send current-model :norm-rows F)
        (send current-model :norm-cols G)))
 
    (defmeth Scatterplot :Reset-Residuals ()
      (let* ((n (send current-model :Nrows))
             (m (send current-model :Ncols))
             (row-mass (send current-model :row-mass))
             (col-mass (send current-model :col-mass))
             (F (send current-model :norm-rows))
             (G (send current-model :norm-cols))
             (dims (send dimension-list :selection))
             (fitted nil)
             (residual nil)
             (n-pts (send residual-plot :num-points))
             (pt-st (send residual-plot :point-state (iseq n-pts))))
        (setf fitted (%* (diagonal row-mass) 
                         (select F (iseq n) dims)
                         (transpose (select G (iseq m) dims))
                         (diagonal col-mass)))
        (setf residual (- (send current-model :Deviation) Fitted))
        (send current-model :Residuals residual)
        (send Residual-plot :clear-points :draw nil)
        (send Residual-plot :add-points
              (combine (row-list (send current-model :deviation)))
              (combine (row-list (send current-model :residuals))) :draw nil)
        (send Residual-plot :adjust-to-data)
        (send Residual-plot :x-axis t t 2)
        (send Residual-plot :y-axis t t 2)
        (send residual-plot :point-state (iseq n-pts) pt-st)
        (send Residual-plot :abline 0 0)))
 
    (defmeth Scatterplot :Reset-Fitplot (pt)
      (let ((A (first (send current-model :gsvd)))
            (B (third (send current-model :gsvd)))
            (frame (send fit-plot :content-rect))
            (num-ticks (third (send fit-plot :x-axis)))
            (Du nil)
            (Inertia nil))
        (cond
          ((= (send current-model :norm-opt) 0)
           (setf Du (%* (inverse (%* (transpose A) A))
                       (transpose A)
                       (diagonal (send current-model :Row-Mass))
                       (send current-model :Norm-Rows))))
          ((= (send current-model :norm-opt) 1)
           (setf Du (%* (inverse (%* (transpose B) B))
                       (transpose B)
                       (diagonal (send current-model :Col-Mass))
                       (send current-model :Norm-Cols)))))
        (setf Inertia (^ (diagonal Du) 2))
        (send fit-plot :start-buffering)
        (send fit-plot :clear-lines)
        (send fit-plot :add-lines 
              (iseq 1 (send current-model :maxdim))
              (^ (second (send current-model :gsvd)) 2))
        (send fit-plot :add-lines
              (iseq 1 (send current-model :maxdim)) Inertia
              :type 'dashed)
        (send fit-plot :adjust-to-data :draw nil)
        (send fit-plot :range 0 1 (send current-model :maxdim))
        (send fit-plot :x-axis t t num-ticks)
        (send fit-plot :y-axis t t 2)
        (send fit-plot :buffer-to-screen)))
 
 
    (setf HideRow-item (send menu-item-proto :new "Hide Row Points"
                        :action
                        #'(lambda ()
                            (send Scatterplot :Show-or-Hide-Row-Pts))))
 
    (send Scatterplot :add-slot 'Hide-Rows)
    (defmeth Scatterplot :Hide-Rows (&optional (show t set))
      (if set (setf (slot-value 'Hide-Rows) show))
      (slot-value 'Hide-Rows))
 
    (defmeth Scatterplot :Show-or-Hide-Row-Pts ()
      (let* ((n (send current-model :Nrows)))
        (send Scatterplot :Hide-Rows
              (not (send Scatterplot :Hide-Rows)))
        (cond
          ((send self :Hide-Rows)
           (send self :point-state (iseq n) 'invisible)
           (send HideRow-Item :title "Show Row Points")
           (send Spinplot :point-masked (iseq n) (repeat t n))
           (send Spinplot :adjust-to-data)
           (send Rows-&-Cols :point-masked (iseq n) (repeat t n))
           (send Rows-&-Cols :redraw))
          (t
           (send self :point-state (iseq n) 'normal)
           (send HideRow-Item :title "Hide Row Points")
           (send Spinplot :point-masked (iseq n) (repeat nil n))
           (send Spinplot :adjust-to-data)
           (send Rows-&-Cols :point-masked (iseq n) (repeat nil n))
           (send Rows-&-Cols :redraw)
           ))
        (send self :adjust-to-data :draw nil)
        (send self :x-axis t t 2)
        (send self :y-axis t t 2)))
    
    (setf HideCol-item (send menu-item-proto :new "Hide Column Points"
                        :action
                        #'(lambda ()
                            (send Scatterplot :Show-or-Hide-Col-Pts))))
 
    (send Scatterplot :add-slot 'Hide-Cols)
    (defmeth Scatterplot :Hide-Cols (&optional (show t set))
      (if set (setf (slot-value 'Hide-Cols) show))
      (slot-value 'Hide-Cols))
 
    (defmeth Scatterplot :Show-or-Hide-Col-Pts ()
      (let* ((n (send current-model :Nrows))
             (m (send current-model :Ncols)))
        (send Scatterplot :Hide-Cols
              (not (send Scatterplot :Hide-Cols)))
        (cond
          ((send self :Hide-Cols)
           (send self :point-state (iseq n (- (+ n m) 1)) 'invisible)
           (send HideCol-Item :title "Show Column Points")
           (send Spinplot :point-masked (iseq n (- (+ n m) 1))
                                        (repeat t m))
           (send Spinplot :adjust-to-data)
           (send Rows-&-Cols :point-masked (iseq n (- (+ n m) 1))
                                           (repeat t m))
           (send Rows-&-Cols :redraw))
          (t
           (send self :point-state (iseq n (- (+ n m) 1)) 'normal)
           (send HideCol-Item :title "Hide Column Points")
           (send Spinplot :point-masked (iseq n (- (+ n m) 1))
                                        (repeat nil m))
           (send Spinplot :adjust-to-data)
           (send Rows-&-Cols :point-masked (iseq n (- (+ n m) 1))
                                           (repeat nil m))
           (send Rows-&-Cols :redraw)))
        (send self :adjust-to-data :draw nil)
        (send self :x-axis t t 2)
        (send self :y-axis t t 2)))
 
    (send Scatterplot :new-menu "Scatterplot"
          :items '(mouse resize-brush dash
                   symbol color selection dash))
    (send (send Scatterplot :menu)
          :append-items HideRow-Item HideCol-Item)
    
    (defmeth Scatterplot :close ()
      (send Spinplot :close))))
 
 
(defmeth Corresp-proto :CRS-Name-List ()
  (setf Rows-&-Cols (name-list (combine (send self :labels)
                                     (send self :variables))
                            :show nil
                            
                            :location location13
                            :title "Rows & Columns"))
  (send Rows-&-Cols :use-color t)
  (send Rows-&-Cols :point-color (iseq (send self :NRows)) 'blue)
  (send Rows-&-Cols :linked t)
  (send Rows-&-Cols :new-menu "List" 
        :items '(ERASE-SELECTION FOCUS-ON-SELECTION SHOW-ALL
                                 COLOR SELECTION))
  (defmeth Rows-&-Cols :do-select-click (x y m1 m2)
    (call-next-method x y m1 m2)
    (let* ((nr (send current-model :NRows))
           (nc (send current-model :NCols))
           (np (send self :num-points))
           (s  (send self :selection)))
      (if (= (length s) 2)
          (when (or (and (< (first s) nr) (>= (second s) nr))
                    (and (>= (first s) nr) (< (second s) nr)))
                (send Residual-plot :selection 
                      (list (identify-residual s nr nc)))
                (send Residual-plot :point-color 
                      (send Residual-plot :selection) 'black)
                (send Residual-plot :redraw t))
          (send Residual-plot :unselect-all-points))))
 
#-msdos(send Rows-&-Cols :size (- (select namelist-size 0) msdos-fiddle)
             (- (select namelist-size 1) msdos-fiddle))
  (send Rows-&-Cols :show-window)
#+msdos(send Rows-&-Cols :size (- (select namelist-size 0) msdos-fiddle)
             (- (select namelist-size 1) msdos-fiddle))
  (defmeth Rows-&-Cols :close ()
    (send Spinplot :close)))
 
(defmeth Corresp-proto :CRS-Residual-Plot ()
  (let ((resid nil))
    (if (not (send current-model :Point-Moved-p))
        (setf resid (send self :residuals))
        (setf resid (send self :o-residuals)))
  (setf Residual-Plot (plot-points
                       (combine (row-list (send self :deviation)))
                       (combine (row-list resid))
                       :show nil
                       :size graph-size
                       :location location21
                       :title "Residual Plot"
                       :variable-labels (list "ObsF - ExpF" "Residual")))
  (send Residual-plot :abline 0 0)
  (send Residual-plot :use-color t)
  (send Residual-plot :x-axis t t 2)
  (send Residual-plot :y-axis t t 2)
  (send Residual-plot :show-window)
  (let* ((nr (send self :nrows))
         (nc (send self :ncols)))
    (defmeth Residual-plot :do-select-click (x y m1 m2)
      (when (not (or (send Scatterplot :Hide-Rows)
                     (send Scatterplot :Hide-Cols)))
            (call-next-method x y m1 m2)
            (when (send self :selection)
                  (send spinplot :selection
                        (locate-residual (first (send self :selection)) nr nc))
                  (send self :selection (list (first (send self :selection)))))
      (if (not (send self :selection))
          (send spinplot :show-all-points)))))
  (send Residual-Plot :new-menu "Residual-Plot" 
        :items '(ERASE-SELECTION FOCUS-ON-SELECTION SHOW-ALL 
                 SYMBOL COLOR SELECTION DASH RESCALE OPTIONS))
  (defmeth Residual-plot :close ()
    (send Spinplot :close))))
 
 
(defmeth Corresp-proto :CRS-Fit-Plot ()
  (setf Fit-Plot (plot-lines (iseq 1 (send self :maxdim))
                             (^ (second (send self :gsvd)) 2)
                             :show nil
                             :size graph-size
                             :location location22
                             :title "Fit Plot"
                             :variable-labels (list "Dimension" "Inertia")))
  (send Fit-Plot :menu nil)
  (send Fit-Plot :range 0 1 (send self :maxdim))
  (send Fit-Plot :x-axis t t (send self :maxdim))
  (send Fit-Plot :y-axis t t 2)
  (send Fit-Plot :show-window)
  (defmeth Fit-Plot :close ()
    (send Spinplot :close)))
 
 
(defmeth Corresp-proto :CRS-Dimension-List ()
  (setf Dimension-list (name-list 
                        (mapcar #'(lambda (x) (format nil "Dimension ~a" x))
                                (+ (iseq (send self :maxdim)) 1))
                        :show nil
                        :size (list (- (first graph-size) msdos-fiddle)
                                    (second graph-size))
                        :location location23
                        :title "Dimensions"))
  (send Dimension-list :selection (iseq (send current-model :dimension)))
  (send Dimension-list :cursor 'finger)
  (send Dimension-list :menu nil)
 
  (defmeth Dimension-list :do-select-click (x y m1 m2)
    (call-next-method x y m1 m2)
    (let* ((Current-Vars (send self :selection))
           (Dim (length Current-Vars)))
      (when (> Dim 0)
            (if (not (send current-model :Point-Moved-p))
                (send self :Recompute-and-Replot-Residuals Current-Vars)
                (send Scatterplot :Reset-Residuals)))
      (when (> Dim 1)
            (send Scatterplot :current-variables
                  (first Current-Vars)
                  (second Current-Vars)))
      (when (> Dim 2)
            (send Spinplot :current-variables
                  (first Current-Vars)
                  (second Current-Vars)
                  (third Current-Vars))
            (send Spinplot :variable-label (iseq (send self :num-points))
                  (send Dimension-list :set-spinplot-labels))
            (send Spinplot :redraw t))))
 
  (defmeth Dimension-list :set-spinplot-labels ()
    (let* ((dummy (repeat "" (send self :num-points)))
           (selected-axes (select (send self :selection) (iseq 3))))
      (setf (select dummy selected-axes)
            (mapcar #'(lambda (x) (format nil "Dim~a" x))
                    (+ 1 selected-axes)))
      dummy))
 
  (defmeth Dimension-list :Recompute-and-Replot-Residuals (dims)
    (let* ((gsvd (send current-model :gsvd))
           (Ar (apply #'bind-columns
                      (select (column-list (first gsvd)) dims)))
           (Dr (diagonal (select (second gsvd) dims)))
           (Br (apply #'bind-columns
                      (select (column-list (third gsvd)) dims)))
           (fitted (%* Ar Dr (transpose Br)))
           (resid (- (send current-model :Deviation) fitted))
           (n-pts (send residual-plot :num-points))
           (pt-st (send residual-plot :point-state (iseq n-pts)))
           )
      (if (= (length (send self :selection))
             (send current-model :maxdim))
          (send current-model :Residuals
                (make-array (list (send current-model :Nrows)
                                  (send current-model :Ncols))
                            :initial-element 0))
          (send current-model :Residuals resid))
      (send Residual-plot :clear-points :draw nil)
      (send Residual-plot :add-points
            (combine (row-list (send current-model :deviation)))
            (combine (row-list (send current-model :residuals))) :draw nil)
      (send Residual-plot :adjust-to-data)
      (send Residual-plot :x-axis t t 2)
      (send Residual-plot :y-axis t t 2)
      (send residual-plot :point-state (iseq n-pts) pt-st)))
 
#-msdos(send Dimension-list :size (- (select namelist-size 0) msdos-fiddle)
             (- (select namelist-size 1) msdos-fiddle))
  (send Dimension-list :show-window)
#+msdos(send Dimension-list :size (- (select namelist-size 0) msdos-fiddle)
             (- (select namelist-size 1) msdos-fiddle))
 
  (defmeth Dimension-list :close ()
    (send Spinplot :close)))
 
    
(defmeth Corresp-proto :visualize ()
  (if (not (eq current-object self)) (setcm self))
  (send current-model :compute-residuals)
  (send current-model :CRS-Spin-plot)
  (send current-model :CRS-Scatter-plot)
  (send current-model :CRS-Name-List)
  (send current-model :CRS-Residual-Plot)
  (send current-model :CRS-Fit-Plot)
  (send current-model :CRS-Dimension-List))
 
 
;---------------------------------------------------------------
;  Supporting Functions
;---------------------------------------------------------------
 
(defun locate-residual (x nr nc)
  (let* ((x (+ x 1)))
    (if (> (mod x nc) 0)
        (let* ((row (+ (div x nc) 1))
               (col (+ nr (- x (* (- row 1) nc)))))
          (list (- row 1) (- col 1)))
        (let* ((row (div x nc))
               (col (+ nr nc)))
          (list (- row 1) (- col 1))))))
 
(defun identify-residual (x nr nc)
  (+ (* (first x) nc) (- (second x) nr)))
 
(defun div (x a)
  (floor (/ x a)))
 
(defun mod (x a)
  (- (/ x a) (div x a)))
 
(defun Get-GSVD (gsvd nr nc)
  (let* ((q (length (second gsvd)))
         (A (select (first gsvd) (iseq nr) (iseq 1 (- q 1))))
         (Du (diagonal (select (second gsvd) (iseq 1 (- q 1)))))
         (B (select (third gsvd) (iseq nc) (iseq 1 (- q 1)))))
    (list A (diagonal Du) B (fourth gsvd))))
 
;-----------------------------
;  Constructor Function
;-----------------------------
(defun Correspondence-Analysis
   (&key
    (data current-data)
    (title "Correspondence Analysis")
    (name (concatenate 'string "CA-" (send current-data :name)))
    (dialog nil)
    (profile "Both")
;forrest changed next line from dimension to dimensions
    (dimensions 3)
    (precision 4))
  (if (not (eq current-object data)) (setcd data))
  (when (= 1 (send current-data :active-nvar '(numeric)))
        (error (format nil "Cannot do Correspondence Analysis with data that has only one active numeric variable.")))
  (let ((Analyze t)
        (norm-opt 2))
  (when (not dialog)
        (cond
          ((equalp (string-downcase profile) "Both")   (setf norm-opt 2))
          ((equalp (string-downcase profile) "Row")    (setf norm-opt 0))
          ((equalp (string-downcase profile) "Column") (setf norm-opt 1))
          (t (error-message (format nil "Error: Invalid normalization method."))
             (setf Analyze nil)))
;forrest changed next line from dimension to dimensions
        (when (< dimensions 1)
              (error-message "Error: Minimum Dimensionality is 1")
              (setf Analyze nil)))
  (when (> 0 (min (send current-data :active-data '(numeric))))
        (error-message (format nil "Error: Data contains negative value(s)."))
        (setf Analyze nil))
  (if Analyze
;forrest changed next line from dimension to dimensions
      (send Corresp-proto :new norm-opt Dimensions Precision 2 Data title name dialog))))
 

(defmeth corresp-proto :isnew (norm-opt dimension precision &rest args)
  (send self :Model-Abbrev "CRS")
  (send self :Norm-opt norm-opt)
  (send self :Dimension dimension)
  (send self :Precision precision)
  (apply #'call-next-method args))
 
;;                       -- End of Part II --
