;;########################################################################
;; mdsmob2.lsp
;; Multidimensional Scaling, Part 2
;; Copyright (c) 1991, 92 by Mary M. McFarlane
;; Written   by Mary McFarlane & Chris Wiesen, September, 1991
;; Rewritten by Mary McFarlane, February, 1992
;;########################################################################

(require "vista")

(defmeth cmds-model :mds-2d-plot (&optional (stream t))
  (setf stressp (plot-points (list 1) 
                            (list (send self :stress))
                             :show nil 
                             
                             :location location23
                             :title "Stress Plot"
                             :variable-labels (list "Iteration" "Stress")))
  (send stressp :range 1 0 .6)
  (send stressp :range 0 0 20)
  (send stressp :y-axis t t 4)
  (send stressp :x-axis t t 5)
  (send stressp :add-lines (list 0 1) (list (send self :stress) 
                                            (send self :stress)))
  (send stressp :size 
        (- (select namelist-size 0) msdos-fiddle) 
        (- (select namelist-size 1) msdos-fiddle))
  (send stressp :show-window)
  (defmeth stressp :close () (send sm :close))

  (setf pp (plot-points (col (send self :xmatrix) 
                             (iseq (send self :number-dimensions)))
               :show nil 
               :scale-type 'fixed
               :location location21
               :size graph-size
               :title "Stimulus Plane"
               :variable-labels (list "Dimension 1" "Dimension 2" 
                     "Dimension 3" "Dimension 4" "Dimension 5" "Dimension 6" 
                     "Dimension 7" "Dimension 8" "Dimension 9")
               :point-labels (send self :stimulus-names)))
  (send pp :linked t)
  (send pp :add-mouse-mode 'relocate-points
        :title "Relocate Point"
        :click :do-nothing
        :cursor 'cross)
  (send pp :add-mouse-mode 'point-moving
      :title "Drag Point"
      :cursor 'finger
      :click :do-point-moving)
  (send pp :adjust-to-data)
  (send pp :x-axis t t 0)
  (send pp :y-axis t t 0)
  (send pp :show-window)
  (defmeth pp :close () (send sm :close))
  (defmeth pp :do-point-moving (x y a b)
    (let* ((pti      (send self :drag-point x y))
           (xvaryvar (send self :content-variables))
           (xvar     (first  xvaryvar))
           (yvar     (second xvaryvar))
           (x        (send self :point-coordinate xvar pti))   
           (y        (send self :point-coordinate yvar pti)))
      (send current-object :use-new-point pti xvar yvar x y)))
  
  )

(defmeth cmds-model :mds-3d-scatterplot (&optional (stream t))
  (setf sm (scatterplot-matrix (col (send self :xmatrix) (iseq (send self :number-dimensions)))
                               :show nil
                      :title "Stimulus Space Planes"
                      :location location11
                      :size graph-size
                      :variable-labels (list "D1" "D2" "D3" "D4" 
                                             "D5" "D6" "D7" "D8" "D9")                           
                      :point-labels (send self :stimulus-names)
                      :scale-type 'fixed))
  (send sm :linked t)
  (send sm :adjust-to-data)
  (send sm :show-window)

  (defmeth sm :do-variable-focus (x y m1 m2)
"Method to focus on variable subplots modified to work for mds-model. Assumes that two other plots exist.  These plots MUST be named pp (the scatterplot) and sp (the spin-plot).  A click on a subplot sends it to scatterplot.  Two or three shift-clicks send first three selected variables to spin-plot and first two to scatterplot."
    (let ((cur-var (send self :current-variables))
          )
      (when (not m1)
            (when (/= (select cur-var 0) (select cur-var 1))
                  (send pp :current-variables 
                        (select cur-var 0) (select cur-var 1) )
                  )
            (setf spin-var ())
            )
      (when m1
            (when (= (select cur-var 0) (select cur-var 1))
                  (setf cur-var (list (select cur-var 0))))
            (when (< (length spin-var) 3)
                  (setf spin-var (adjoin (select cur-var 0) spin-var))
                  (when (< (length spin-var) 3)
                        (if (equal (length cur-var) 2)
                            (setf spin-var 
                                  (adjoin (select cur-var 1) spin-var)))))
            (when (= (length spin-var) 3)
                  (setf spin-var (reverse spin-var))
                  (send sp 
                        :current-variables (select spin-var 0)
                        (select spin-var 1) (select spin-var 2)
                        :draw nil)
                  (send pp :current-variables 
                        (select spin-var 0) (select spin-var 1) )
                  
                  (send sp :transformation nil :draw nil)
                  (send sp :adjust-to-data)
                  (let ((cur-var (send sp :current-variables))
                        )
                    (send sp :set-variables-with-labels cur-var
                          (select (send sp :variable-labels) cur-var))
                    )
                  
                  (setf spin-var ())
                  )
            )
      ))

  (send sm :add-mouse-mode 'focus-on-variables
        :title "Focus On Variables"
        :click :do-variable-focus
        :cursor 'finger)

  (send sm  :mouse-mode 'focus-on-variables)
  )

(defmeth cmds-model :mds-3d-spin-plot (&optional (stream t))
      (setf sp (spin-plot 
                (col (send self :xmatrix) 
                     (iseq (send self :number-dimensions)))
                :show nil 
                :location location12
                :size graph-size
                :title "Stimulus Space"
                :variable-labels 
                (list "D1" "D2" "D3" "D4" "D5" "D6" "D7" "D8" "D9")
                :point-labels (send self :stimulus-names)
                :scale-type 'fixed))
            (let* ((h (+ (send sp :text-ascent)
                         (send sp :text-descent))))
              (send sp :margin 0 (round (* 1.5 h)) 0 0))
            (setf stim-overlay (send graph-overlay-proto :new))
            (let* ((ascent (send sp :text-ascent))
                   (x ascent)
                   (y (round (* 1.5 ascent)))
                   (box ascent))
              (send stim-overlay :add-slot 'location
                    (list x y box (round (+ x (* 1.5 box))))))
            (send sp :add-overlay stim-overlay)
            (send sp :linked t)
            (send sp :margin 0 36 0 18)
            (defmeth stim-overlay :location () (slot-value 'location))
            (defmeth stim-overlay :redraw ()
              (let* ((loc (send stim-overlay :location))
                     (x (first loc))
                     (y (second loc))
                     (box (third loc))
                     (string-x (fourth loc))
                     (graph (send stim-overlay :graph)))
                (send graph :frame-rect x (- y box) box box)
                (send graph :draw-string "Iterate" string-x y)
                (send graph :frame-rect (+ 68 x) (- y box) box box)
                (send graph :draw-string "Back Up" (+ 68 string-x) y)))
            (defmeth stim-overlay :do-click (x y p1 p2)
              (let* ((loc (send stim-overlay :location))
                    (box (third loc))
                     (left (first loc))
                    (top (- (second loc) box))          
                    (right (+ left box))
                    (bottom (+ top box))
                    (graph (send stim-overlay :graph)))
               (when (and (< left x right) (< top y bottom))
                     (let* ((n (array-dimension (send current-model :xmatrix) 0))
                            (m (array-dimension (send current-model :xmatrix) 1)))
                       (send current-model :prev-matrix (matrix (list n m)
                                    (copy-list (combine (send current-model :xmatrix))))))
                                                                 
                     (setf iter-number (get-value-dialog "How many iterations?"
                                                        :initial 5))
                     (dotimes (i (select iter-number 0))
                              (let* ((n (array-dimension (send current-model :xmatrix) 0))
                                     (m (array-dimension (send current-model :xmatrix) 1))
                                     (old (matrix (list n m) 
                                                  (copy-list (combine 
                                                              (send current-model :xmatrix)))))
                                     (new (guttman-transform (send current-model :xmatrix)))
                                     (stress2 (calc-stress new 
                                                           (send current-model
                                                                 :transformed-data))))
                                (send stressp :add-points (list 
                                                           (+ 1 (send stressp :num-points)))
                                      (list stress2))
                                (send stressp :add-lines-with-points)
                                (send stressp :redraw)
                                (send current-model :stress stress2)
                                (send current-model :old-matrix old)
                                (send current-model :xmatrix new))
                              (send current-model :new-sm-points)
                              (send current-model :new-sp-points)
                              (send current-model :new-pp-points)))
                (when (and (and (and (< (+ 68 left) x (+ 68 right)) (< top y bottom)) 
                           (eq nil p1)) (eq nil p2))
                      (send current-model :back-just-one))
                (when (and (and (and (< (+ 68 left) x (+ 68 right)) (< top y bottom)) 
                           (eq nil p1)) (eq t p2))
                      (send current-model :back-to-start))
                (when (and (and (and (< (+ 68 left) x (+ 68 right)) (< top y bottom)) 
                           (eq t p1)) (eq nil p2))
                      (send current-model :undo-iterations))
         
))
      (send sp :show-window)
  (defmeth sp :close () (send sm :close))

  (defmeth sp :set-variables-with-labels (v labs)                                 
    (let ((n (send self :num-variables)))                                   
      (send self :variable-label (iseq n) (repeat "" n))             
      (send self :variable-label v labs)                                           
      (apply #'send self :current-variables v)
      ))
  )

(defmeth cmds-model :name-plot (&optional (stream t))
  (setf np (name-list (send self :stimulus-names)
                 :show nil 
                 
                 :title "Stimuli"
                 :location location13))
  (send np :size 
        (- (select namelist-size 0) msdos-fiddle) 
        (- (select namelist-size 1) msdos-fiddle))
  (send np :show-window)
  (send np :linked t)
  (defmeth np :close () (send sm :close))
  )

(defmeth cmds-model :initialize ()
  (let* ((current-model-menu-length (length (send *model-menu* :items)))
         )
    (send *desktop* :no-menu-marks *model-menu*)
    (setf model-menu-item-name 
         (concatenate 'string  "MDS-" (send current-data :name)))
    (send *model-menu* :append-items
          (send menu-item-proto :new model-menu-item-name :mark t)
          )
    ))

(defmeth cmds-model :visualize ()
  (if (not (eq current-object self)) (setcm self))
  (send current-model :mds-3d-scatterplot)
  (send sm :show-window)
  (send current-model :mds-3d-spin-plot)
  (send current-model :name-plot)
  (send current-model :mds-2d-plot)
  (send current-model :scree-plot)
  (send sp :show-window)
  (defmeth sm :close () 
    (send self :remove)
    (send sp :remove)
    (send pp :remove)
    (send np :remove)
    (send scree-plot :remove)
    (send stressp :remove))
  )

(defmeth cmds-model :create-data 
  (&key (dialog nil)
        (coefs t)
        (input nil))
  (if (not (eq current-object self)) (setcm self))
  (let ((creator (send *desktop* :selected-icon))
        (desires (list (list (if coefs 0) (if input 1))))
        )
    (if dialog
        (setf desires
              (choose-subset-dialog "Choose Desired Data Objects"
                  '("Stimulus Coefficients (Coordinates)"
                    "Analyzed (Averaged) Input Data")
                    :initial (select desires 0))))
    (when desires
          (when (member '0 (select desires 0))
                (send current-model :mds-coefs-data-object  creator))
          (when (member '1 (select desires 0))
                (send current-model :mds-indata-data-object creator)))
    ))

(defmeth cmds-model :mds-coefs-data-object (creator)
  (data (concatenate 'string "Coefs-" (send current-model :name))
   :created creator
   :creator-object self
   :title (concatenate 'string "MDS Coefficients for " 
                       (send self :title))
   :data (combine (select (send self :xmatrix)
                          (iseq (length (send self :stimulus-names)))
                          (iseq (send self :number-dimensions))))
   :variables (mapcar #'(lambda (x) (format nil "Dim~a" x)) 
                      (iseq (send self :number-dimensions)))
   :labels (send self :stimulus-names)
   :types (repeat '"numeric" (send self :number-dimensions)))
  )

(defmeth cmds-model :mds-indata-data-object (creator)
  (data (concatenate 'string "Analyzed-" (send current-model :name))
   :created creator
   :creator-object self
   :matrices '("Analyzed Data")
   :title (concatenate 'string "Data analyzed by MDS from " 
                       (send self :title))
   :data (combine (send self :data))
   :variables (send self :stimulus-names)
   :labels (send self :stimulus-names))
  )

(defun cmds-model-object (&key dissim-object-name 
                               (matrix-number nil)
                               (matrices-numbers nil)
                               (add-constants nil)
                               (number-dimensions nil)
                               (title "Multidimensional Scaling")
                               (mds-2d-plot t)
                               (mds-3d-scatterplot t)
                               (mds-3d-spin-plot t)
                               (name-plot t)
                               (print-cmds-model t))
  (let ((object (send cmds-model :new dissim-object-name matrix-number
                      matrices-numbers add-constants number-dimensions 
                      mds-2d-plot mds-3d-scatterplot mds-3d-spin-plot
                      name-plot print-cmds-model)))
    (send object :menu (concatenate 'string "MDS-" (send current-data :name)))
    (setf current-model object)
    (setf current-object object)
    (send object :title title)
;next line moved to multidimensional scaling constructor function by fwy 4.27
;   (send *toolbox* :copy-tool-icon 3)
    (send *model-menu* :enabled t)
    (send object :add-model-menu-item "MDS")
    (setcm object) ;added to fix menu initialization fwy f.27
    object))                      

(defun mds-options-dialog ()  
  (let* ((d-label (send text-item-proto :new "Dimensions (minimum=3)"))
         (dimensionality-text
          (send edit-text-item-proto :new "3" :text-length 1))
         (cancel (send modal-button-proto :new "Cancel"))
         (ok (send modal-button-proto :new "OK" 
                   :action #'(lambda ()
                       (eval (read (make-string-input-stream
                                          (send dimensionality-text :text))
                                         nil)))
                   ))
         (mds-options-dialog 
          (send modal-dialog-proto :new
                (list (list dimensionality-text d-label)
                      (list ok cancel))
                :default-button ok))
         )
    (send mds-options-dialog :modal-dialog)
    ))

(defmeth cmds-model :use-new-point (i xvar yvar xscore yscore )
  (let* ((scores (list xscore yscore))
         (cur-vars (list xvar yvar))
         (new-matrix (send current-model :xmatrix))
         (n (array-dimension (send current-model :xmatrix) 0))
         (m (array-dimension (send current-model :xmatrix) 1))
         (copy-old (matrix (list n m) 
                           (copy-list (combine 
                                        (send current-model :xmatrix))))))
  (send current-model :old-matrix copy-old)
    (if (eq (send pp :mouse-mode) 'relocate-points)
        (setf k (select i 0))
        (setf k i))
    (send pp :point-coordinate xvar k xscore)
    (send pp :point-coordinate yvar k yscore)
    (send pp :redraw)
    
    (send sp :point-coordinate xvar k xscore)
    (send sp :point-coordinate yvar k yscore)
    (send sp :redraw)
    (send sm :point-coordinate xvar k xscore)
    (send sm :point-coordinate yvar k yscore)
    (send sm :redraw)
  (dotimes (j 2)
             (setf (aref new-matrix
                         k
                         (select cur-vars j))
                   (select scores j)))
    (send current-model :xmatrix new-matrix)
    (send stressp :add-points (list (+ 1 (send stressp :num-points)))
          (list (calc-stress new-matrix (send current-model :transformed-data)))
          :draw nil)
    (send stressp :add-lines-with-points)
    (send stressp :redraw)))

(defun multidimensional-scaling 
  (&key 
   (data current-data)
   (title "Multidimensional Scaling")
   (name nil)
   (dialog nil)
   (dimensions 3)
   )
"ViSta function to perform Multidimensional Scaling.  
With no arguments, performs a 3-dimensional scaling using all active data matrices in the current data. Keyword arguments are:
:DIMENSIONS followed by an integer to specify the dimensionality of the analysis (default - and minimum - is 3);
:DATA followed by the name of the data to be analyzed (default: current-data);
:TITLE followed by a character string (default: Multidimensional Scaling);
:DIALOG followed by t (to display parameters dialog box) or nil (default)."
  (let ((object nil)
        )
    (if (not (eq current-object data)) (setcd data))
    (if (not name) (strcat "MDS-" (send current-data :name)))
;next statement moved from cmds-model constructor function fwy 4.27
    (send *toolbox* :copy-tool-icon 3)
    (if dialog (setf dimensions (mds-options-dialog)))
    (cond
      ((not dimensions)
       (send *toolbox* :reset-button 4))
      ((>= dimensions 3)
       (setf object 
             (cmds-model-object 
              :dissim-object-name data
              :matrices-numbers 
              ($position (send current-data :active-matrices '(all))
                         (send current-data :matrices)) 
              :title title
              :add-constants t
              :number-dimensions dimensions)))
      (t
       (send *toolbox* :reset-button 4)
       (error "Minimum dimensionality is 3.")))
    object))
