(defmeth table-data-object-proto :data-matrix ()
"Message args: none
Reports table data as a multivariate category matrix."
  (let* ((way-names (send self :ways))
         (class-names (send self :classes))
         (ncells (send self :ncells))
         (cellfreqs (send self :cellfreqs))
         (nways (send self :nways))
         (class-num (repeat 0 nways))
         (nobs (send self :nobs))
         (nclasses (send self :nclasses))
         (mv-data (make-array (list nobs (1+ nways)))) 
         (response (combine (send self :data)))
         (L 0)
         (string "")) 
    (dotimes (i ncells)
             (dotimes (k (select cellfreqs i))
                 
                      (setf (select mv-data L 0) (select response L))
                      (dotimes (j nways)
                               (setf (select mv-data L (1+ j))
                                     (select (select class-names j)
                                             (select class-num j))))
                      (setf L (1+ L)))
             (dolist (j (iseq (- nways 1) 0))
                     (setf (select class-num j) (+ (select class-num j) 1))
                     (if (= (select class-num j)  (select nclasses j))
                         (setf (select class-num j) 0)
                         (return))))
    mv-data))

(defmeth table-data-object-proto :create-data (&optional name) 
"Message args: (&optional name)
Creates a multivariate data object from the current table data.  The data object is named NAME (a string) if specified, otherwise a dialog is presented for name. Returns object identification of the new data object."
  (if (not (eq current-object self)) (setcd self))
  (setf merge-dob nil)
  (let ((menu-name nil)
        )
    (if name
        (setf menu-name name)
        (setf menu-name 
              (get-string-dialog "Create Classification Data Named:"
                        :initial (strcat "Class-" (send self :name)))))
    (when (and menu-name (= (length menu-name) 0)) 
          (error-message "error - you must specify a name."))
    (cond 
      ((and menu-name (> (length menu-name) 0))
       (data menu-name
             :created (send *desktop* :selected-icon)
             :title (concatenate 'string "Created from "(send self :title))
             :data (combine (send current-data :data-matrix))
             :variables (combine (send current-data :variables)
                                 (send current-data :ways))
             :labels (repeat (send current-data :labels) 
                             (send current-data :cellfreqs))
             :types (combine "Numeric" 
                             (repeat "Category" (send current-data :nways)))
             )))))